#!/usr/bin/perl

my $module = "sfs1";
my $users = 2;

if ($#ARGV > 0) { die "usage: prog [users]\n"; }
if ($#ARGV == 0) { $users = int($ARGV[0]); }

#
# first machine
#

my $mach0  = "jfk1-gblx.ron.lcs.mit.edu";
my $home0  = "/users/home/benjie/pastwatch.dist";
my $keyf0   = "$home0/exp/$mach0/key0";
my $root0  = "/users/home/benjie/.pastwatch";
my $w0     = "$home0/exp/$mach0/w0";
my $past0  = "$home0/bin/pastwatch";
my $lsd0   = "216.165.108.39:32221";
my $dir0   = "$home0/exp/";
my $rpcw0  = 64;
my $base   = "$home0/exp/base";   # only used for import

#
# second machine
#

my $mach1  = "ana1-gblx.ron.lcs.mit.edu";
my $home1  = "/users/home/benjie/pastwatch.dist";
my $keyf1   = "$home1/exp/$mach1/key1";
my $root1  = "/users/home/benjie/.pastwatch";
my $w1     = "$home1/exp/$mach1/w1";
my $past1  = "$home1/bin/pastwatch";
my $lsd1   = "65.164.24.58:32221";
my $dir1   = "$home1/exp/";
my $rpcw1  = 64;

system ("date");

# first, create a view

system ("rm -rf ~/.pastwatch/test");
my $o = `setenv CFSRW_CONNTIMER 30; export CFSRW_CONNTIMER=30; setenv LSD_TCPSOCKET $lsd0; export LSD_TCPSOCKET=$lsd0; pastview test`;

my $view;
my $keyh0;
my $keyh1;
my @allkeys;

if ($o =~ /New view: (\S+)/) {
  $view = $1;
  print "view: $view\n";
}
if ($o =~ /New keyhash: (\S+)/) {
  $keyh0 = $1;
  print "key0: $keyh0\n";
}

$o = `setenv CFSRW_CONNTIMER 30; export CFSRW_CONNTIMER=30; setenv LSD_TCPSOCKET $lsd0; export LSD_TCPSOCKET=$lsd0; pastview -j $view test`;
if ($o =~ /New keyhash: (\S+)/) {
  $keyh1 = $1;
  print "key1: $keyh1\n";
}

for (my $x=2; $x < $users; $x++) {
  $o = `setenv CFSRW_CONNTIMER 30; export CFSRW_CONNTIMER=30; setenv LSD_TCPSOCKET $lsd0; export LSD_TCPSOCKET=$lsd0; pastview -j $view test`;
  if ($o =~ /New keyhash: (\S+)/) {
    print " key: $1\n";
    push @allkeys, $1;
  }
}

for (my $x=0; $x <= $#allkeys; $x++) {
  system ("rm -rf /tmp/pasttmp; mkdir /tmp/pasttmp; cd /tmp/pasttmp; setenv CFSRW_CONNTIMER 30; export CFSRW_CONNTIMER=30; setenv LSD_TCPSOCKET $lsd0; export LSD_TCPSOCKET=$lsd0; pastwatch -v $view -s ~/.pastwatch/test/key.$allkeys[$x] import -m test test$x");
}

system ("scp ~/.pastwatch/test/key.$keyh0 $mach0:$keyf0");
system ("scp ~/.pastwatch/test/key.$keyh1 $mach1:$keyf1");

# then, start working on two separate client machines

ssh ($mach0, "rm -rf $w0; mkdir -p $w0");
ssh ($mach1, "rm -rf $w1; mkdir -p $w1");

ssh ($mach0, "setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; $past0 -v $view -s $keyf0 status a");
ssh ($mach1, "setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; $past1 -v $view -s $keyf1 status a");

ssh ($mach0, "echo Import; cd $base; rm -rf pastwatch.core ; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; time $past0 -v $view -s $keyf0 import -m checkin $module; echo Done Import");

ssh ($mach0, "sleep 15; echo Checkout Hot; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; time $past0 -v $view -s $keyf0 checkout -d $w0/$module $module; echo Done Checkout Hot");

ssh ($mach1, "sleep 15; echo Checkout Cold; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; time $past1 -v $view -s $keyf1 checkout -d $w1/$module $module; echo Done Checkout Cold");

sub doit {
  my ($w, $diffs) = @_;

  if ($w == 0) {
    ssh ($mach0, "sleep 5; cd $dir0; echo ===== $diffs ======; grep -c Index $diffs; wc $diffs");
    ssh ($mach0, "cd $w0; patch -f < $dir0/$diffs");
    ssh ($mach0, "cd $w0; /bin/rm -rf `find $module -print | grep \".orig\" | xargs`");
    ssh ($mach0, "cd $w0; /bin/rm -rf `find $module -print | grep \".rej\" | xargs`");
    ssh ($mach0, "cd $w0; /bin/rm -rf $w0/zzz; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; $past0 -v $view -s $keyf0 update $module > $w0/zzz");
    ssh ($mach0, "cd $w0; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; $dir0/add-past.pl $view $keyf0 $past0 $w0/zzz");
    ssh ($mach0, "echo Commit 0; cd $w0; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; time $past0 -v $view -s $keyf0 commit -m $diffs $module; echo Done Commit 0");
    ssh ($mach0, "cd $w0; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; $past0 -v $view -s $keyf0 update $module");
    ssh ($mach1, "sleep 5; echo Update Warm 1; cd $w1; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; time $past1 -v $view -s $keyf1 update $module; echo Done Update Warm 1");
  }
  
  else {
    ssh ($mach1, "sleep 5; cd $dir1; echo ===== $diffs ======; grep -c Index $diffs; wc $diffs");
    ssh ($mach1, "cd $w1; patch -f < $dir1/$diffs");
    ssh ($mach1, "cd $w1; /bin/rm -rf `find $module -print | grep \".orig\" | xargs`");
    ssh ($mach1, "cd $w1; /bin/rm -rf `find $module -print | grep \".rej\" | xargs`");
    ssh ($mach1, "cd $w1; /bin/rm -rf $w1/zzz; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; $past1 -v $view -s $keyf1 update $module > $w1/zzz");
    ssh ($mach1, "cd $w1; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; $dir1/add-past.pl $view $keyf1 $past1 $w1/zzz");
    ssh ($mach1, "echo Commit 1; cd $w1; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; time $past1 -v $view -s $keyf1 commit -m $diffs $module; echo Done Commit 1");
    ssh ($mach1, "cd $w1; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root1; setenv LSD_TCPSOCKET $lsd1; setenv CFSRW_RPCWIN $rpcw1; $past1 -v $view -s $keyf1 update $module");
    ssh ($mach0, "sleep 5; echo Update Warm 0; cd $w0; setenv CFSRW_CONNTIMER 30; setenv PASTWATCH_ROOT $root0; setenv LSD_TCPSOCKET $lsd0; setenv CFSRW_RPCWIN $rpcw0; time $past0 -v $view -s $keyf0 update $module; echo Done Update Warm 0");
  }
}

sub ssh {
  my($h, $cmd) = @_;
  my $c = "ssh -x -a $h '$cmd'";
  my $status = system($c);
  return $status;
}

system ("sleep 5; date");

doit (0, "patch-200");
doit (1, "patch-201");
doit (0, "patch-202");
doit (1, "patch-203");
doit (0, "patch-204");
doit (1, "patch-205");
doit (0, "patch-206");
doit (1, "patch-207");
doit (0, "patch-208");
doit (1, "patch-209");
doit (0, "patch-210");
doit (1, "patch-211");
doit (0, "patch-212");
doit (1, "patch-213");
doit (0, "patch-214");
doit (1, "patch-215");
doit (0, "patch-216");
doit (1, "patch-217");
doit (0, "patch-218");
doit (1, "patch-219");
doit (0, "patch-220");
doit (1, "patch-221");
doit (0, "patch-222");
doit (1, "patch-223");
doit (0, "patch-224");
doit (1, "patch-225");
doit (0, "patch-226");
doit (1, "patch-227");
doit (0, "patch-228");
doit (1, "patch-229");
doit (0, "patch-230");
doit (1, "patch-231");
doit (0, "patch-232");
doit (1, "patch-233");
doit (0, "patch-234");
doit (1, "patch-235");
doit (0, "patch-236");
doit (1, "patch-237");
doit (0, "patch-238");
doit (1, "patch-239");

system ("date");

