#include "record.h"
#include "cfsw_record.h"
#include "mklog.h"
#include "cfsrw.h"

bigint
mklog::make_block_id (const void *data, int size) 
{
  long *rd = (long *)data;
  char id[sha1::hashsize];
  sha1_hash (id, rd, size);
    
  bigint ID;
  mpz_set_rawmag_be (&ID, id, sizeof (id));
  return ID;
}

void 
mklog::done (cfsrw_status status) 
{
  ptr<mklog> t = _self;
  _self = NULL; // Cleanup this object after this procedure ends.

  assert (_final_cb);
  _final_cb (status);
}

void
mklog::store_block_cb (callback<void, bigint, int>::ref cb,
		       ns_stat status, bigint key)
{
  if (status != NS_OK) {
    warn << "store_block_cb failed\n";
    done (CFSRWERR_IO);
  }
  cb (key, 1);
}

void
mklog::store_cb (bool ispk, bigint id, int ok)
{
  if (_verbose && ispk) {
    char raw[20];
    mpz_get_rawmag_be (raw, sizeof (raw), &id);
    str name = armor64A (raw, sizeof (raw));
    strbuf out;
    out << name << "\n";
    out.tosuio ()->output (1);
  }
  
  _work--;
  if (_work == 0)
    done (CFSRW_OK);
}

bigint
mklog::store (bigint va, const str &data, 
	      callback<void, bigint, int>::ref cb)
{
  return nsobj->insert_block
    (data.cstr(), data.len(), wrap(this, &mklog::store_block_cb, cb));
}

void
mklog::store_lptr (bigint va, const str &data,
		   callback<void, bigint, int>::ref cb, ptr <sfspriv> sk)
{
  assert (sk);
  sfs_sig2 s;
  sk->sign (&s, data);
  sfs_pubkey2 k;
  sk->export_pubkey (&k);
  nsobj->insert_pk (sk, data, wrap (this, &mklog::store_block_cb, cb));
}

void
mklog::create_log (ptr <sfspriv> sk, bigint va, bool root, str timestamp, 
		   bool verbose, callback<void, cfsrw_status>::ref final_cb)
{
  _self = mkref(this);
  _verbose = verbose;
  _final_cb = final_cb;
  bigint cid = 0;
  bigint head = ns::pkid (sk);
 
  if (root) {
    cfsw_sattr attr;
    attr.valid_mode = 1;
    attr.mode = 0755;
    attr.valid_size = 1;
    attr.size = 0;
    inode_delta r (CFSW_DIR, attr);

    _work++;
    bigint rid = store (va, r.serialize (), 
			wrap (this, &mklog::store_cb, false));

    commit_record c;
    //      c._proposed_name = 
    c._head = head;
    c._prev = 0;
    c._ndeltas = 1;
    c._deltas.push_back (rid);
    if (timestamp && timestamp != "") {
      time_t t = get_date ((char*) timestamp.cstr());
      c._time = t;
    }

    _work++;
    cid = store (va, c.serialize (),
		 wrap (this, &mklog::store_cb, false));
  }

  log_pointer p;
  p._pw_seqn = 2;
  p._commit = cid;
  p._lock = 0;
  p._verf = 0;

  _work++;
  store_lptr (va, p.serialize (), 
	      wrap (this, &mklog::store_cb, true), sk);
}

mklog::mklog () 
{
  _work = 0;
}

