/*
 *  Copyright (C) 2002-2004  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#include "stdinc.h"
#include "cfsrw.h"
#include "cfsr.h"

#define CONCURRENT_READS 64
				
class cfsrw_read_obj
{
  cbcfsrw cb;
  int fd;
  bigint fh;
  uint64 size;

  uint64 off;
  unsigned int reads;
  int error;
  ptr<snapshot_obj> ss;

  void read_reply (uint64 off, uint64 cnt, cfsrcallp c, cfsrw_status status);
  bool do_read();
  void done ();

public:
  cfsrw_read_obj (int d, bigint h, uint64 sz, ptr<snapshot_obj> s, cbcfsrw c);
};

void
cfsrw_read_file (int fd, bigint fh, uint64 size, ptr<snapshot_obj> sobj,
                 cbcfsrw cb)
{
  vNew cfsrw_read_obj (fd, fh, size, sobj, cb);
}

void
cfsrw_read_obj::done () 
{
  if (reads == 0) {
    if (error)
      cb (CFSRWERR_IO);
    else
      cb (CFSRW_OK);
    delete this;
  }
}

void
cfsrw_read_obj::read_reply
(uint64 off, uint64 cnt, cfsrcallp c, cfsrw_status status)
{
  reads --;

  if (status || error) {
    error = 1;
    done ();
    return;
  }

  cfsr_read_res *res = static_cast<cfsr_read_res *> (c->res ());
      
  if (res->data->len != cnt) {
    warn << "asked for " << off << "+" << cnt
         << ", got " << res->data->len << "\n";
    error = 1;
    done ();
    return;
  }

  pwrite (fd, res->data->data, res->data->len, off);
  do_read ();
  done ();
}

bool
cfsrw_read_obj::do_read ()
{
  if (off >= size)
    return true;

  uint64 cnt = size - off;
  cnt = cnt > SN_BLKSIZE ? SN_BLKSIZE : cnt;
  reads ++;

  cfsrcallp c = New refcounted<cfsrcall>
    (CFSRPROC_READ, wrap (this, &cfsrw_read_obj::read_reply, off, cnt));
  cfsr_read_arg *arg = static_cast<cfsr_read_arg *> (c->arg ());
  arg->file = fh;
  arg->off = off;
  arg->cnt = cnt;
  off += cnt;

  cfsr::dispatch (ss, c);
  return false;
}

cfsrw_read_obj::cfsrw_read_obj (int d, bigint h, uint64 sz,
                                ptr<snapshot_obj> s, cbcfsrw c)
  : cb (c)
{
  fd = d;
  fh = h;
  size = sz;
  reads = 0;
  error = 0;
  ss = s;

  if (ftruncate (fd, 0) < 0)
    error = 1;
  else {
    off = 0;
    bool eof = false;
    for (int i = 0; i < CONCURRENT_READS && !eof; i++)
      eof = do_read ();
  }
  done ();
}

