#ifndef _LOCTABROUTE_H_
#define _LOCTABROUTE_H_

#include "chord.h"
#include "chord_types.h"
#include "bigint.h"
#include "id_utils.h"
#include "chord_impl.h"
#include <ihash.h>
#include "misc_utils.h"
#include "id_utils.h"

class loctabroute : public vnode_impl {
 protected:
  int nsucc;

  void verify_nsucc();
  void find_route (const chordID &x, cbroute_t cb);
  void find_route (const chordID &x, const bigint projkey, cbroute_t cb);
  void find_route_hop_cb (cbroute_t cb, route_iterator *ri, bool done);
  void find_succlist_hop_cb (cbroute_t cb, route_iterator *ri, u_long m, bool done);
  void find_successor_cb (chordID x, cbroute_t cb, vec<chord_node> s, 
			  route search_path, chordstat status);

 public:
  static ref<vnode> produce_vnode (str blacklist_dir, ref<chord> _chordnode, 
				   ref<rpc_manager> _rpcm,
				   ref<location> _l);

  loctabroute (ref<chord> _chordnode, 
	       ref<rpc_manager> _rpcm,
	       ref<location> _l);

  loctabroute (str blacklist_dir, ref<chord> _chordnode, ref<rpc_manager> _rpcm,
	       ref<location> _l);

  virtual ~loctabroute (void);
  
  virtual ptr<route_iterator> produce_iterator (chordID xi);
  virtual ptr<route_iterator> produce_iterator (chordID xi,
						const rpc_program &uc_prog,
						int uc_procno,
						ptr<void> uc_args);
  virtual route_iterator *produce_iterator_ptr (chordID xi);
  virtual route_iterator *produce_iterator_ptr (chordID xi,
						const rpc_program &uc_prog,
						int uc_procno,
						ptr<void> uc_args);

  virtual ptr<route_iterator> produce_iterator (chordID xi, bigint projkey);
  virtual ptr<route_iterator> produce_iterator (chordID xi, bigint projkey,
						const rpc_program &uc_prog,
						int uc_procno,
						ptr<void> uc_args);
  virtual route_iterator *produce_iterator_ptr (chordID xi, bigint projkey);
  virtual route_iterator *produce_iterator_ptr (chordID xi, bigint projkey,
						const rpc_program &uc_prog,
						int uc_procno,
						ptr<void> uc_args);

  virtual void find_successor (const chordID &x, cbroute_t cb);
  virtual void find_successor (const chordID &x, const bigint projkey, cbroute_t cb);
  virtual void find_succlist (const chordID &x, u_long m, cbroute_t cb,
			      ptr<chordID> guess = NULL);
  virtual void find_succlist (const chordID &x, const bigint projkey, u_long m, cbroute_t cb,
			      ptr<chordID> guess = NULL);


  virtual ptr<location> my_pred () const;
  virtual ptr<location> my_succ () const;
  virtual ptr<location> my_pred (bigint projkey);
  virtual ptr<location> my_succ (bigint projkey);

  virtual ptr<location> closestpred (const chordID &x, const vec<chordID> &f);
};

#endif
