#ifndef KEYHASH_TREE_H
#define KEYHASH_TREE_H

#include "qhash.h"
#include "skiplist.h"
#include "lru.h"
#include "id_utils.h"
#include "dhash_blockstore.h"

struct khwrap {
  sklist_entry<khwrap> hlink;
  bigint n;
  khwrap (bigint n) : n (n) {}
};

typedef skiplist<khwrap, bigint, &khwrap::n, &khwrap::hlink> khtab;
  
struct lwrap {
  sklist_entry<lwrap> hlink;
  bigint n;
  bool offered;
  unsigned offertime;
  bool notified;
  ptr<khtab> t;
  lwrap (bigint n)
    : n (n), offered (false), notified (false)
  { t = New refcounted<khtab> (); }
};

class keyhash_idx 
{
private:
  lwrap *_syncnext;
  lrucache<bigint, long, hashID> _vidx;
  skiplist<lwrap, bigint, &lwrap::n, &lwrap::hlink> _lidx;
  ptr<dhash_blockstore> _storage;

  void lidx_add (bigint projkey, bigint key);
  int lidx_remove (bigint projkey, bigint key);

public:
  keyhash_idx (ptr<dhash_blockstore> storage);

  dhash_stat insert (ptr<dbrec> key, ptr<dbrec> val);
  dhash_stat insert (bigint projkey, ptr<dbrec> key, ptr<dbrec> val);
  ptr<dbrec> lookup (bigint n);
  ptr<dbrec> lookup (bigint n, bigint &lk);
  int remove(bigint removeme);

  bool getver (bigint n, long &);
  bool getprojkey (bigint n, bigint &l);

  struct offer_state {
    bigint projkey;
    vec<bigint> keys;
    vec<long> vers;
  };

  void notify (bigint k, bool set);
  ptr<offer_state> offernext (bigint pred, bigint my);
  ptr<offer_state> findkeys (bigint pred, bigint my);
};

#endif

