/*
 *  Copyright (C) 2005  Massachusetts Institute of Technology
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */

#ifndef PROJINTERFACE_H
#define PROJINTERFACE_H

#include "stdinc.h"
#include "ns.h"
#include "vervec.h"
#include "ptr_block.h"
#include "memberlist.h"

struct head_state {
  vervec pkv;      // log pointers, and their revision numbers
  vec<lptr> lptrs; // log pointers, sorted by their DHT addresses
  comvec commits;  // list of the current log heads.  

  void clear() {
    pkv.clear();
    commits.clear();
    lptrs.clear();
  }
  void print() {
    pkv.print();
    commits.print();
    for(unsigned i=0; i<lptrs.size(); i++) {
      warn << "lptrs -> " << armor64A(lptrs[i]->commit()) << "\n";
    }
  }
};

class projinterface {
  /*
   * This class provides convenient read access to the
   * memberlist and user blocks.
   */

 public:
  projinterface(bigint projectid);
  void fetch_heads(callback<void, ptr<head_state>, log_status>::ref cb);
  /* After fetch_heads finishes, it sets the <head_state> contents.
   *   - head_state.commits is the current state of the log pointers.  
   *     There's one entry for each log, and each entry contains the
   *     address of the log pointer, and the address of the log's 
   *     current head.
   */
  
  unsigned nusers () const;
  bigint get_user_id (unsigned i) const;
  str get_user_label (unsigned i) const;
  long get_userblk_version (unsigned i) const;

  unsigned nlogs() const;
  bigint get_log_ptr(unsigned i) const;
  str get_log_label(unsigned i) const;
  long get_log_dhash_version (unsigned i) const;
  int get_log_owner(unsigned i) const; // returns the index of the owner of this log.

  str get_notify() const;
  long get_projblk_version () const;

 protected:
  
  ptr<memberlist> _memberlist;
  vec< ptr<ptr_block> >  _userblocks;

  ptr<head_state> _headstate;
  vec<bigint>     _lptr_addrs;
  vec<str>        _labels;
  vec<long>       _dhash_seqns;

  vec<int>           _log_to_user; // maps the log index to the owning user's index

  unsigned _work;
  log_status _status;
  bigint _projectid;
  callback<void, ptr<head_state>, log_status>::ptr _final_cb;

  void read_memberlist_cb(log_status status);
  void read_userblock_cb(unsigned i, log_status status);
  void read_lptr_cb (bigint lptr_addr, unsigned i, unsigned j, 
		     lptr u, long pkv, ns_stat status);
  void load_commit_cb(bigint lptr_addr, lptr u, unsigned i, unsigned j, 
		      crec c, ns_stat status);
  void check_done();
  void add_lptr(bigint new_addr, lptr new_lptr, str new_label, 
		long new_dhashseq);

};
#endif
