#include "merkle_tree.h"
#include "misc_utils.h"
#include "merkle_misc.h"
#include <sys/time.h>
#include "id_utils.h"

void
dump_db  (dbfe *db)
{
  warn << "db_dump\n";
  ptr <dbEnumeration> it = db->enumerate ();
  ptr<dbPair> entry = it->firstElement ();
  while (entry) {
    merkle_hash key = to_merkle_hash (entry->key);
    warn << "db_dump: " << key << "\n";
    entry = it->nextElement ();
  }
  it = NULL;
}
void
dump_keys(dbfe *db) {
  warn << "db_dump\n";
  ptr <dbEnumeration> it = db->enumerate ();
  ptr<dbPair> entry = it->firstElement ();
  while (entry) {
    warn << "  " << armor64A(dbrec2id(entry->key)) 
	 << " 0x" << hexdump(entry->key->value, entry->key->len) << "\n";
    entry = it->nextElement ();
  }
  it = NULL;
}

#if 0
merkle_tree::merkle_tree (ptr<dhash_blockstore> _storage)
{
  unsigned int i;
  initializing = true;
  vec<ptr<dbrec> > blocks;
  storage = _storage;
  bigint projkey = 0;

  // initialize dummy db
  // dummydb is kept consistent with the merkle tree.
  // The storage database is in the background, but is not necessarily consistent
  // with the merkle tree because other processes may add/remove blocks.
  dummydb = New refcounted<dbfe> ();
  dbOptions opts;
  opts.addOption("opt_cachesize", 1000);
  opts.addOption("opt_nodesize", 4096);
  if (int err = dummydb->opendb(NULL, opts))
    fatal << "merkle_tree::merkle_tree opendb failed: " << strerror(err);
  dummydata = New refcounted<dbrec>("", 1);

  timeval tv0, tv1;
  gettimeofday(&tv0, NULL);
  storage->getchblockids(&blocks);
  gettimeofday(&tv1, NULL);
  warn << "start: " << tv0.tv_sec << " " << tv0.tv_usec << "\n";
  warn << "end:   " << tv1.tv_sec << " " << tv1.tv_usec << "\n";

  if (blocks.size() > 0)
    warn << "Database is not empty.  Loading into " << blocks.size() << " keys into merkle tree\n";

  for(i=0; i<blocks.size(); i++) {
    block b (to_merkle_hash (blocks[i]), dummydata);
    //storage->ch_get_projkey(blocks[i], &projkey);
    assert( !insert (projkey, 0, &b, &root) );
  }
  initializing = false;
  check_invariants();
}
#endif

merkle_tree::merkle_tree (ptr<dhash_blockstore> _storage, chordID min, 
			  chordID max, const vec<bigint> &exclude_proj)
{
  unsigned int i;
  initializing = true;
  vec<ptr<dbrec> > blocks;
  storage = _storage;
  bigint projkey = 0;

  // initialize dummy db
  // dummydb is kept consistent with the merkle tree.
  // The storage database is in the background, but is not necessarily consistent
  // with the merkle tree because other processes may add/remove blocks.
  dummydb = New refcounted<dbfe> ();
  dbOptions opts;
  opts.addOption("opt_cachesize", 1000);
  opts.addOption("opt_nodesize", 4096);
  if (int err = dummydb->opendb(NULL, opts))
    fatal << "merkle_tree::merkle_tree opendb failed: " << strerror(err);
  dummydata = New refcounted<dbrec>("", 1);

  timeval tv0, tv1;
  gettimeofday(&tv0, NULL);
  storage->getchblockids(&blocks, id2dbrec(min), id2dbrec(max), 0);
  gettimeofday(&tv1, NULL);

  //if (blocks.size() > 0) warn << "Database is not empty.  Range has " << blocks.size() << " keys\n";

  ihash<bigint, exclusion_entry, &exclusion_entry::id, &exclusion_entry::link, hashID> excl_hash;
  for(unsigned i=0; i<exclude_proj.size(); i++)
    excl_hash.insert(New exclusion_entry(exclude_proj[i]));

  for(i=0; i<blocks.size(); i++) {
    block b (to_merkle_hash (blocks[i]), dummydata);
    storage->ch_get_projkey(blocks[i], &projkey);
    if (!excl_hash[projkey])
      assert( !insert (projkey, 0, &b, &root) );
  }
  initializing = false;
  check_invariants();
  excl_hash.deleteall();
}


void
merkle_tree::rehash (u_int depth, const merkle_hash &key, merkle_node *n)
{
  n->hash = 0;
  if (n->count == 0)
    return;
  
  sha1ctx sc;
  if (n->isleaf ()) {
    assert (n->count > 0 && n->count <= 64);
    merkle_hash prefix = key;
    prefix.clear_suffix (depth);
    ///warn << "prefix: " << prefix << "\n";
    vec<merkle_hash> keys = database_get_keys_dummy (dummydb, depth, prefix);

    for (u_int i = 0; i < keys.size (); i++)
      sc.update (keys[i].bytes, keys[i].size);
  } else {
    for (int i = 0; i < 64; i++) {
      merkle_node *child = n->child (i); 
      sc.update (child->hash.bytes, child->hash.size);
      ///warn << "INTE: update " << child->hash << "\n";
    }
  }
  sc.final (n->hash.bytes);
  ///warn << "final: " << n->hash << "\n";
}


void
merkle_tree::count_blocks (u_int depth, const merkle_hash &key,
			   array<u_int64_t, 64> &nblocks)
{
  for (int i = 0; i < 64; i++)
    nblocks[i] = 0;
  
  // XXX duplicates code with rehash ()
  merkle_hash prefix = key;
  prefix.clear_suffix (depth);
  vec<merkle_hash> keys = database_get_keys_dummy (dummydb, depth, prefix);

  for (u_int i = 0; i < keys.size (); i++) {
    u_int32_t branch = keys[i].read_slot (depth);
    nblocks[branch] += 1;
  }

}


void
merkle_tree::leaf2internal (u_int depth, const merkle_hash &key, 
			    merkle_node *n) 
{
  assert (n->isleaf ());
  array<u_int64_t, 64> nblocks;
  count_blocks (depth, key, nblocks);
  n->leaf2internal ();
  
  merkle_hash prefix = key;
  prefix.clear_suffix (depth);
  
  u_int xmax = (depth == merkle_hash::NUM_SLOTS) ? 16 : 64;
  for (u_int i = 0; i < xmax; i++) {
    // warn << "leaf2internal [" << i << "] = " << nblocks[i] << "\n";
    
    merkle_node *child = n->child (i);
    child->initialize (nblocks[i]);
    prefix.write_slot (depth, i);
    rehash (depth+1, prefix, child);
  }
}

int
merkle_tree::insert (bigint projkey, u_int depth, block *b, merkle_node *n)
{
  int ret = 0;

  if (n->isleaf () && n->leaf_is_full ())
    leaf2internal (depth, b->key, n);
  
  if (n->isleaf ()) {
    if (!initializing) {
      ret = database_insert (projkey, storage, b);
      if (ret) warn << "Warning: could not insert block " << b->key 
		    << " into disk database\n";
    }
    ret = dummydb->insert(todbrec(b->key), dummydata);

    if (ret != 0)
      n->count -= 1;
  } else {
    u_int32_t branch = b->key.read_slot (depth);
    ret = insert (projkey, depth+1, b, n->child (branch));
  }
  
  n->count += 1;
  rehash (depth, b->key, n);
  return ret;
}

int
merkle_tree::insert (bigint projkey, block *b)
{
  if (dummydb->lookup(todbrec(b->key))) {
    // XXX This should not happen often because the blocks are removed 
    //   from merkle before they are removed from the disk database.
    //   It only happens in a race condition where the block B is removed
    //   by the auditor at the same time as B being inserted via 
    //   merkle_tree::insert. This will leave the merkle tree with a key
    //   that is not present in the disk database.

   warn << "merkle_tree::insert: key already exists in merkle_tree! " 
	 << b->key << "\n";
    return database_insert (projkey, storage, b);
  }

  return insert (projkey, 0, b, &root);
}

#if 0
void
merkle_tree::remove (block *b, bool full_remove)
{
  // assert block must exist..
  if (!dummydb->lookup(todbrec(b->key))) {
    warn << "merkle_tree::remove: key does not exist in merkle_tree " 
	 << b->key << "\n";
    // XXX maybe we should check if it's in the disk database, and remove it there
    return;
  }

  remove (0, b, &root, full_remove);
}
#endif

// XXX later, lookup will need to check if the block is in the 
//  the disk database if it's not in the merkle_tree

merkle_node *
merkle_tree::lookup (u_int *depth, u_int max_depth, 
		     const merkle_hash &key, merkle_node *n)
{
  // recurse down as much as possible
  if (*depth == max_depth || n->isleaf ())
    return n;
  u_int32_t branch = key.read_slot (*depth); 
  //the [6*depth, 6*(depth +1) bits determine which branch to follow
  // for a given key
  *depth += 1;
  return lookup (depth, max_depth, key, n->child (branch));
}

// return the node a given depth matching key
// returns NULL if no such node exists
merkle_node *
merkle_tree::lookup_exact (u_int depth, const merkle_hash &key)
{
  u_int realdepth = 0;
  merkle_node *n = lookup (&realdepth, depth, key, &root);
  assert (realdepth <= depth);
  return  (realdepth != depth) ? NULL : n;
}

// Might return parent.
// Never returns NULL.
// Never returns a node deeper than depth.
merkle_node *
merkle_tree::lookup (u_int depth, const merkle_hash &key)
{
  u_int depth_ignore = 0;
  return lookup (&depth_ignore, depth, key, &root);
}

merkle_node *
merkle_tree::lookup (u_int *depth, u_int max_depth, const merkle_hash &key)
{
  *depth = 0;
  return lookup (depth, max_depth, key, &root);
}

// return the deepest node whose prefix matches key
// Never returns NULL
merkle_node *
merkle_tree::lookup (const merkle_hash &key)
{
  return lookup (merkle_hash::NUM_SLOTS, key);
}

void
merkle_tree::dump ()
{
  root.dump (0);
}

void
merkle_tree::check_invariants ()
{
  root.check_invariants (0, 0, dummydb);
}

void
merkle_tree::stats_helper (uint depth, merkle_node *n)
{
  stats.nodes_per_level[depth]++;
  stats.num_nodes++;

  if (n->isleaf ()) {
    if (n->count == 0) {
      stats.empty_leaves_per_level[depth]++;
      stats.num_empty_leaves++;
    }
    stats.leaves_per_level[depth]++;
    stats.num_leaves++;
  } else {
    stats.internals_per_level[depth]++;
    stats.num_internals++;
  }

  if (! n->isleaf ()) {
    for (uint i = 0; i < 64; i++) {
      merkle_node *child = n->child (i); 
      stats_helper (depth+1, child);
    }
  }
}


void
merkle_tree::compute_stats ()
{
  bzero (&stats, sizeof (stats));
  stats_helper (0, &root);
  
  warn.fmt ("      %10s %10s %10s %10s\n", "leaves", "MT leaves", "internals", "nodes");

  // dont print the trailing zeroes...
  uint cutoff = max_depth - 1;
  while (cutoff && !stats.nodes_per_level[cutoff])
    cutoff--;
  for (uint i = 0; i <= cutoff; i++) {
    warn.fmt ("%4d: %10d %10d %10d %10d\n",
	      i,
	      stats.leaves_per_level[i],
	      stats.empty_leaves_per_level[i],
	      stats.internals_per_level[i],
	      stats.nodes_per_level[i]);
  }
  warn << "-----------------------------------------\n";
  warn.fmt ("total %10d %10d %10d %10d\n",
	    stats.num_leaves, stats.num_empty_leaves, stats.num_internals, stats.num_nodes);
  assert (stats.num_leaves > 0);

  uint64 mn = max_depth;
  uint64 mx = 0;
  double ave = 0;
  for (uint i = 0; i < max_depth; i++) {
    if (stats.leaves_per_level[i] == 0)
      continue;

    ave += i * stats.leaves_per_level[i];
    mn = i < mn ? i : mn;
    mx = i > mx ? i : mx;
  }
  ave /= stats.num_leaves;

  warn << "depth min " << mn << "\n";
  warn << "depth max " << mx << "\n";
  char buf[2000];
  snprintf (buf, sizeof (buf), "depth ave %f\n", ave);
  warn << buf;

  warn << "blocks: " << root.count << "\n";
  snprintf (buf, sizeof (buf), 
	    "blocks/node: %f\n"
	    "blocks/leaf: %f\n"
	    "blocks/non-empty-leaf: %f\n"
	    "blocks/internal: %f\n",
	    root.count / (float)stats.num_nodes,
	    root.count / (float)stats.num_leaves,
	    root.count / (float)(stats.num_leaves - stats.num_empty_leaves),
	    root.count / (float)stats.num_internals);
  warn << buf;
}
