%#include <chord_types.h>
%#include <dhash_types.h>

/*  -------------------------------------------------------------------------- 
 *  The DHASHGATEWAY_PROGRAM is the very narrow interface between clients of 
 *  dhash (such as chordcd, sfsrodb, dbm) and the lsd process.  Since this RPC 
 *  interface is intended to be transported over a unix domain socket, blocks 
 *  are retrieved/inserted in their entirety, ie. are not broken into chunks. 
 */

/* if we get many more optional arguments consider switching to union */
struct dhash_insert_arg {
  chordID blockID;        /* the key */
  chordID projkey;
  dhash_ctype ctype;
  dhash_value block;
  int32_t len;
  int options;
};

struct dhash_retrieve_arg {
  chordID blockID;
  chordID projkey;
  dhash_ctype ctype;
  int options;
};

struct dhash_retrieve_resok {
  dhash_value block;
  dhash_ctype ctype;
  int hops;
  int errors;
  int retries;
  int32_t len;
  u_int32_t times<>;
  chordID path<>;
};

struct dhash_insert_resok {
  chordID path<>;
};

union dhash_insert_res switch (dhash_stat status) {
 case DHASH_OK:
   dhash_insert_resok resok;
 case DHASH_BUSY:
   void;
 default:
   void;
};

union dhash_retrieve_res switch (dhash_stat status) {
 case DHASH_OK:
   dhash_retrieve_resok resok;
 case DHASH_BUSY:
   void;
 default:
   void;
};


struct dhashgw_retrieve_pk_arg {
  chordID key;
  chordID projkey;
  unsigned vers;
  int options;
};

struct dhashgw_check_pks_arg {
  chordID projkey;
  chordID keys<>;
  unsigned versions<>;
};

struct dhashgw_check_pks_resok {
  chordID keys<>;
  unsigned versions<>;
};

union dhashgw_check_pks_res switch (dhash_stat status) {
 case DHASH_OK:
   dhashgw_check_pks_resok resok;
 case DHASH_BUSY:
   void;
 default:
   void;
};

struct dhashgw_send_stats_arg {
  dhash_value stats;
  int32_t len;
};

union dhashgw_send_stats_res switch (dhash_stat status) {
  case DHASH_OK:
    void;
  case DHASH_ERR:
    void;
  default:
    void;
};


program DHASHGATEWAY_PROGRAM {
  version DHASHCLNT_VERSION {
    void DHASHPROC_NULL (void) = 0;
    dhash_insert_res DHASHPROC_INSERT (dhash_insert_arg) = 1;
    dhash_retrieve_res DHASHPROC_RETRIEVE (dhash_retrieve_arg) = 2;
    dhash_retrieve_res DHASHPROC_RETRIEVEPK (dhashgw_retrieve_pk_arg) = 4;
    dhashgw_check_pks_res DHASHPROC_CHECKPKS (dhashgw_check_pks_arg) = 5;
    dhashgw_send_stats_res DHASHPROC_SEND_STATS (dhashgw_send_stats_arg) = 6;
  } = 1;
} = 444448;

