#ifndef _ADMCONTROL_H_
#define _ADMCONTROL_H_
/*
 *
 * Copyright (C) 2003 Alexander Yip (yipal@lcs.mit.edu)
 *
 *  Permission is hereby granted, free of charge, to any person obtaining
 *  a copy of this software and associated documentation files (the
 *  "Software"), to deal in the Software without restriction, including
 *  without limitation the rights to use, copy, modify, merge, publish,
 *  distribute, sublicense, and/or sell copies of the Software, and to
 *  permit persons to whom the Software is furnished to do so, subject to
 *  the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include "chord_types.h"
#include "crypt.h"
#include "sfscrypt.h"
#include "skiplist.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif

/*
 * The hostlist is a singleton object for using the hostlist.
 * The file format looks like this:
 *    0xa939d092d3f03cfa643f5efa55d8448a0559ccfa05f5be758a3b61d
 *    125f3b6d9e795cb5e5d3326a786e9fc94f4e66e4272a3b79d8728cd86
 *    7d2777f0684db43a5420492edf9807df92d4cb724f4639a14d51a7ca8
 *    5301facc4b84c3de57022de3d9500eb9dc16c9ef25b54ea3ff938a60c
 *    b714d1f9e77155c42107b10bf91de71cc7b62115ad5e98735a2067dd7
 *    eb4ee356538e1e4db72c1d2beb988eae517e5
 *    Version 2
 *    18.26.4.77 2000 0xaa8f158015946e2e5aeac8bf479225ccb39b374 fission.lcs.mit.edu
 * 
 * The first line contains a signature of remaining lines, signed
 * by the Aqua administrator's private key. The first two columns 
 * are IP address and port number. The third column is the Chord 
 * ID of the host. The last column is ignored. It is a comment for the line.
 */

struct hostentry {
  sklist_entry<hostentry> hlink;

  chord_hostname name;
  int32_t port;
  chordID id;
  ptr<sfspub> verifier;
};


class hostlist {
  str raw_hostlist;
  str hostlist_file;
  int version;
  chordID my_id;
  ptr<sfspub> verifier;
  skiplist<hostentry, chordID, &hostentry::id, &hostentry::hlink> hosts;

  void sign(str s);
  int verify_sig(str signedhostlist);
  hostentry *insert_host (chord_hostname name, int32_t port, chordID id, ptr<sfspub> pub);

  struct hostentry* find (chordID id);

 public:

  static hostlist &only ();

  // return 1 if successfully parsed entire hostlist
  // return 0 if could not parse hostlist at all
  // return -1 if partially parsed hostlist
  static int parse_hostlist(str signedlist, int *out_version,
			    vec<chord_hostname> *names, vec<int32_t> *ports, 
			    vec<chordID> *ids, vec<ptr<sfspub> > *verifiers);


  hostlist();
  ~hostlist() {}

  void set_myid( chordID id );
  chordID get_myid();
  int get_version() {return version;}
  str get_raw_hostlist() {return raw_hostlist;}
  int read_hostlist (str filename);
  int read_verifierpk (str filename);
  void write_hostlist(str newhostlist);

  bool initialized();


  // Returns -1 if invalid, otherwise, return hostlist version.
  int verify_hostlist(str signedhostlist);

  bool exists (chord_hostname name, int32_t port, chordID id, int vnode = 0);
  chordID get_chordID (str addr, int port, int vnode = 0);
  unsigned int size() {return hosts.size();}
  struct hostentry* find (str name, int32_t port);

  hostentry *first();
  hostentry *next(hostentry *current);
  hostentry *next_circular(hostentry *current);
};




#endif /* _ADMCONTROL_H_ */
