#include "penviron.h"
#include "sfsmisc.h"
#include "async.h"
#include "str.h"

static char* defaults[] = {
  "PASTWATCH_TCPSOCKET", DEFAULT_PW_TCPSOCKET,
  "LSD_TCP_TIMEOUT", "10",
  "PASTWATCH_CONNTIMER", "10",
  "CFSRW_RPCWIN", "32",
  "CFSRW_RPCTIMER", "180",
  "PASTWATCH_DEBUG_CONN", "0"
};
static unsigned ndefaults = 6;


char* pgetenv(const char *name) {
  unsigned i;
  char* s = getenv(name);

  if (!s) {
    for(i=0; i<ndefaults; i++)
      if (!strcmp(name, defaults[i*2]))
	s = defaults[i*2 + 1];
  }

  return s;
}

str pget_pwroot() {

  char* pwhome = getenv("PASTWATCH_ROOT");
  if (pwhome) 
    return pwhome;

  // use default location
  str home = getenv("HOME");
  if (!home)
    fatal << "Could not find HOME environment variable.";

  strbuf b;
  b << home << "/.pastwatch";
  return str(b);
}

str pget_replicadir() {
  
  char* replica_dir = getenv("PASTWATCH_LOCAL_REPLICA");
  char* user = getenv("USER");

  if (replica_dir) 
    return replica_dir;
  else if (!user) 
    fatal << "If PASTWATCH_LOCAL_REPLICA is not defined, then you\n"
	  << "must define the USER environment variable to use the\n"
	  << "default replica location.\n";
  else {
    //fatal << "Environment variable PASTWATCH_LOCAL_REPLICA is undefined.  Please set PASTWATCH_LOCAL_REPLICA.  See the pastwatch(1) man page for more details.\n";
    str s = "/tmp/" << str(user) << "-pastreplica";
    return s;
  }
}

str pget_statlog() {
  char* statlog = getenv("PASTWATCH_STATS_LOG");
  if (statlog)
    return str(statlog);

  str s = pget_replicadir() << "/stats.log";
  return s;
}
